//+------------------------------------------------------------------+
//|                                                           mladen |
//|                                                  gaussian filter |
//+------------------------------------------------------------------+
#property copyright "Copyleft  2008, mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame = "Current time frame";
extern int    Length    = 20;
extern int    Order     = 2;
extern int    Price     = PRICE_CLOSE;
extern bool   Interpolate  = true;

//
//
//
//
//

double buffer[];
double bufferda[];
double bufferdb[];
double slope[];
double coeffs[][3];
string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);      
   SetIndexBuffer(0,buffer); 
   SetIndexBuffer(1,bufferda);
   SetIndexBuffer(2,bufferdb);
   SetIndexBuffer(3,slope); SetIndexDrawBegin(0,Length);
      Length = MathMax(Length,2);
      Order  = MathMax(Order,1);
         InitCoeffs(Length,Order);

   //
   //
   //
   //
   //

      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars";
      timeFrame         = stringToTimeFrame(TimeFrame);
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//


int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { buffer[0] = limit+1; return(0); }
         if (timeFrame != Period())
         {
            limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            if (slope[limit]==-1) CleanPoint(limit,bufferda,bufferdb);
            for(int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,timeFrame,Time[i]);               
                  buffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",Length,Order,Price,0,y);
                  bufferda[i] = EMPTY_VALUE;
                  bufferdb[i] = EMPTY_VALUE;
                  slope[i]    = iCustom(NULL,timeFrame,indicatorFileName,"",Length,Order,Price,3,y);

                  if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

                  //
                  //
                  //
                  //
                  //

                  datetime time = iTime(NULL,timeFrame,y);
                     for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
                     for(int m = 1; m < n; m++)
                        buffer[i+m] = buffer[i] + (buffer[i+n] - buffer[i]  ) * m/n;
            }
            for(i=limit; i>=0; i--) if (slope[i]==-1) PlotPoint(i,bufferda,bufferdb,buffer);
            return(0);
         }

   //
   //
   //
   //
   //
   
   if (slope[limit]==-1) CleanPoint(limit,bufferda,bufferdb);
   for(i = limit; i >= 0; i--)
   {
      buffer[i] = iMA(NULL,0,1,0,MODE_SMA,Price,i)*coeffs[Order][1];
      double sign = 1;
         for (int r=1; r <= Order; r++, sign *= -1.0)
                  buffer[i] += sign*coeffs[r][0]*coeffs[r][2]*buffer[i+r];
                  bufferda[i] = EMPTY_VALUE;
                  bufferdb[i] = EMPTY_VALUE;
                  slope[i]    = slope[i+1];
                  if (buffer[i]>buffer[i+1]) slope[i] =  1;
                  if (buffer[i]<buffer[i+1]) slope[i] = -1;
                  if (slope[i]==-1) PlotPoint(i,bufferda,bufferdb,buffer);
   }
   return(0);
}



//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define Pi 3.141592653589793238462643

void InitCoeffs(double length, int n)
{
   double b = (1.0 - MathCos(2.0*Pi/length))/(MathPow(MathSqrt(2.0),2.0/n) - 1.0);
   double a = -b + MathSqrt(b*b + 2.0*b);

   ArrayResize(coeffs,n+1);
   for(int r=0; r<=n; r++)
   {
       coeffs[r][0] = fact(n)/(fact(n-r)*fact(r));
       coeffs[r][1] = MathPow(    a,r);
       coeffs[r][2] = MathPow(1.0-a,r);
   }
}

//
//
//
//
//

double fact(int n)
{
   double a=1;
         for(int i=1; i<=n; i++) a*=i;
   return(a);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}